/***************************************************************
 * Name:      odeslanedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "odeslanedialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/csv.xpm"

void OdeslaneCtrl::CreateColumns()
{
    InsertColumn(OCOL_CAS, wxString::FromUTF8("Datum a čas"), wxLIST_FORMAT_LEFT, 140);
    InsertColumn(OCOL_FAKTURA, "Faktura", wxLIST_FORMAT_LEFT, 80);
    InsertColumn(OCOL_PRIJEMCE, wxString::FromUTF8("Příjemce"), wxLIST_FORMAT_LEFT, 150);
    setResizeColumn(OCOL_PRIJEMCE);
}

wxString OdeslaneCtrl::OnGetItemText(long item, long column) const
{
    return m_dialog->getItemText(item, column);
}

int OdeslaneCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *OdeslaneCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(OdeslaneDialog, wxDialog)

BEGIN_EVENT_TABLE(OdeslaneDialog, wxDialog)
    EVT_BUTTON(wxID_OK, OdeslaneDialog::OnOk)
    EVT_MENU(ID_ODESLANE_EXPORT, OdeslaneDialog::OnExport)
    EVT_LIST_COL_CLICK(ID_ODESLANE_LIST, OdeslaneDialog::OnColumnClick)
END_EVENT_TABLE()

OdeslaneDialog::OdeslaneDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Seznam odeslaných faktur")), m_sortCol(OdeslaneCtrl::OCOL_DEF_SORT), m_sortAsc(true)
{
    m_odeslane = idbmodel->odeslane();
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_ODESLANE_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, "Exportovat seznam");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new OdeslaneCtrl(this, this, ID_ODESLANE_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    updateList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString OdeslaneDialog::getItemText(long item, long column) const
{
    if(item < 0 || item >= (int)m_odeslane.GetCount())
        return "";
    switch(column)
    {
    case OdeslaneCtrl::OCOL_CAS:
        return m_odeslane[item].cas().Format("%x %X");
    case OdeslaneCtrl::OCOL_FAKTURA:
        return m_odeslane[item].faktura();
    case OdeslaneCtrl::OCOL_PRIJEMCE:
        return m_odeslane[item].prijemce();
    default:
        return "";
    }
}

void OdeslaneDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void OdeslaneDialog::OnExport(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, "Exportovat seznam do", wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << wxString::FromUTF8("Datum a čas") << delim << "Faktura" << delim << wxString::FromUTF8("Pří­jemce") << delim << endl;
    for(size_t i=0; i<m_odeslane.GetCount(); i++)
    {
        text << dxutils::enquotes(m_odeslane[i].cas().Format("%x %X"), delim);
        text << delim;
        text << dxutils::enquotes(m_odeslane[i].faktura(), delim);
        text << delim;
        text << dxutils::enquotes(m_odeslane[i].prijemce(), delim);
        text << delim;
        text << endl;
    }
}

void OdeslaneDialog::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= OdeslaneCtrl::OCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<OdeslaneCtrl::OColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    updateList();
}

DbModel::OSort OdeslaneDialog::columnToOdeslaneSort(OdeslaneCtrl::OColumn column)
{
    switch(column)
    {
    case OdeslaneCtrl::OCOL_CAS: return DbModel::OS_CAS;
    case OdeslaneCtrl::OCOL_FAKTURA: return DbModel::OS_FAKTURA;
    case OdeslaneCtrl::OCOL_PRIJEMCE: return DbModel::OS_PRIJEMCE;
    default: return DbModel::OS_CAS;
    }
}

void OdeslaneDialog::setColumnImage(OdeslaneCtrl::OColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void OdeslaneDialog::updateList()
{
    size_t oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    idbmodel->sortOdeslany(m_odeslane, columnToOdeslaneSort(m_sortCol), m_sortAsc);
    size_t cnt = m_odeslane.GetCount();
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_ODESLANE_EXPORT, true);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_ODESLANE_EXPORT, false);
    }
}
