/***************************************************************
 * Name:      zasobydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "zasobydialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(ZasobyDialog, wxDialog)

BEGIN_EVENT_TABLE(ZasobyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ZasobyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ZasobyDialog::OnCancel)
    EVT_TREE_SEL_CHANGED(ID_ZASOBY_TREE, ZasobyDialog::OnTreeChanged)
END_EVENT_TABLE()

ZasobyDialog::ZasobyDialog(wxWindow *parent, const wxDateTime &datum)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Seznam zásob")), m_datum(datum)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

int ZasobyDialog::zasobaId() const
{
    wxTreeItemId id = m_navigation->GetSelection();
    if(!id) return -1;
    SkladZasobaItemData *sz = dynamic_cast<SkladZasobaItemData*>(m_navigation->GetItemData(id));
    if(!sz) return -1;
    if(!sz->sklad()) return sz->zasobaid();
    return -1;
}

void ZasobyDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxSplitterWindow *splitter = new wxSplitterWindow(this);
    m_navigation = new wxTreeCtrl(splitter, ID_ZASOBY_TREE, wxDefaultPosition, wxSize(150,300), wxTR_SINGLE|wxTR_NO_LINES|wxTR_HAS_BUTTONS);
    wxTreeItemId root = m_navigation->AddRoot("Sklady");
    m_navigation->SetItemData(root, new SkladZasobaItemData(true, -1, -1));
    m_navigation->SetFocus();
    dxSkladArray sklady = idbmodel->sklady();
    for(size_t i=0; i < sklady.GetCount(); i++)
    {
        wxTreeItemId sklad = m_navigation->AppendItem(root, sklady[i].nazev());
        m_navigation->SetItemData(sklad, new SkladZasobaItemData(true, sklady[i].id(), -1));
        dxZasobaArray zasoby = idbmodel->zasobyBySkladId(sklady[i].id());
        for(size_t j=0; j < zasoby.GetCount(); j++)
        {
            wxTreeItemId zasoba = m_navigation->AppendItem(sklad, zasoby[j].nazev());
            m_navigation->SetItemData(zasoba, new SkladZasobaItemData(false, -1, zasoby[j].id()));
        }
    }
    m_navigation->Expand(root);
    m_navigation->SelectItem(root);
    m_page = new HtmlPage(splitter);
    splitter->SplitVertically(m_navigation, m_page);
    m_mainSizer->Add(splitter, 1, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    m_ok = new wxButton(this, wxID_OK, "Vybrat", wxDefaultPosition, wxDefaultSize, 0);
    m_ok->Disable();
    btnSizer->AddButton(m_ok);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void ZasobyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void ZasobyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void ZasobyDialog::OnTreeChanged(wxTreeEvent &event)
{
    wxTreeItemId id = event.GetItem();
    if(!id) return;
    SkladZasobaItemData *sz = dynamic_cast<SkladZasobaItemData*>(m_navigation->GetItemData(id));
    if(!sz) return;
    if(sz->sklad())
    {
        m_page->updateSklad(sz->skladid());
        m_ok->Disable();
    }
    else
    {
        m_page->updateZasoba(sz->zasobaid(), m_datum);
        m_ok->Enable();
    }
}

IMPLEMENT_DYNAMIC_CLASS(HtmlPage, wxPanel)

HtmlPage::HtmlPage(wxWindow *parent)
    : wxPanel(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_html = new wxHtmlWindow(this, wxID_ANY, wxDefaultPosition, wxSize(400,300), wxBORDER_SUNKEN);
    updateSklad(-1);
    m_mainSizer->Add(m_html, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void HtmlPage::updateSklad(int skladid)
{
    wxString page = "<html><body>";
    Sklad sk = idbmodel->getSklad(skladid);
    if(sk.isValid())
    {
        page.Append(wxString::Format("<b>%s</b>", sk.nazev()));
        page.Append("<table>");
        page.Append("<tr>");
        page.Append("<td>Popis:</td>");
        page.Append(wxString::Format("<td>%s</td>", dxutils::toHtmlBr(sk.popis())));
        page.Append("</tr>");
        page.Append("</table>");
    }
    else
    {
        page.Append(wxString::FromUTF8("<b>Seznam skladů a zásob</b>"));
    }
    page.Append("</body></html>");
    m_html->SetPage(page);
}

void HtmlPage::updateZasoba(int zasobaid, const wxDateTime &datum)
{
    wxString page = "<html><body>";
    Zasoba za = idbmodel->getZasoba(zasobaid);
    if(za.isValid())
    {
        page.Append(wxString::Format("<b>%s</b>", za.nazev()));
        page.Append("<table>");
        page.Append("<tr>");
        page.Append("<td>Popis:</td>");
        page.Append(wxString::Format("<td>%s</td>", dxutils::toHtmlBr(za.popis())));
        page.Append("</tr>");
        page.Append("<tr>");
        page.Append(wxString::FromUTF8("<td>Zásoba:</td>"));
        double pocet, celkem;
        idbmodel->getZasobaStav(zasobaid, datum, &pocet, &celkem);
        page.Append(wxString::Format("<td>%.3f %s</td>", pocet, za.mj()));
        page.Append("</tr>");
        page.Append("<tr>");
        page.Append(wxString::FromUTF8("<td>Hodnota zásoby:</td>"));
        page.Append(wxString::Format(wxString::FromUTF8("<td>%.2f Kč</td>"), celkem));
        page.Append("</tr>");
        page.Append("<tr>");
        page.Append(wxString::Format(wxString::FromUTF8("<td>Prodejní cena<br/>za %s:</td>"), za.mj()));
        page.Append(wxString::Format(wxString::FromUTF8("<td>%.2f Kč</td>"), za.cenamj()));
        page.Append("</tr>");
        if(idbmodel->isFirmaPlatce())
        {
            page.Append("<tr>");
            page.Append("<td>Bez DPH:</td>");
            page.Append(za.bezdph() ? "<td>Ano</td>" : "<td>Ne</td>");
            page.Append("</tr>");
            page.Append("<tr>");
            page.Append(wxString::FromUTF8("<td>Sazba daně:</td>"));
            page.Append(wxString::Format("<td>%d%%</td>", za.dph()));
            page.Append("</tr>");
        }
        page.Append("<tr>");
        page.Append(wxString::FromUTF8("<td>Datum poslední<br/>inventury:</td>"));
        page.Append(idbmodel->getInventuraLastDateForZasoba(za.id()).Format("<td>%d.%m.%Y %H:%M:%S</td>"));
        page.Append("</tr>");
        page.Append("</table>");
    }
    page.Append("</body></html>");
    m_html->SetPage(page);
}
