/***************************************************************
 * Name:      fakturaxmlprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/filename.h>
#include "fakturaxmlprint.h"
#include "utils/dxutils.h"
#include "wx/pdfdc.h"
#include "wx/pdffontmanager.h"
#include "config.h"
#include "gui/template/tags.h"

FakturaXmlPrint::FakturaXmlPrint(wxWindow *parent, const Faktura &faktura, const wxString &xml)
    : DocumentPrint(parent, "Faktura"), m_faktura(faktura), m_xml(xml)
{
    m_document = new Document();
    parseXml();
    int posY = m_document->marginTop()-PAGE_OFFSET;
    int sheight = m_document->sectionHeight(HEADER1);
    moveElements(m_document->elementsBySection(HEADER1), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(HEADER2);
    moveElements(m_document->elementsBySection(HEADER2), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(HEADER3);
    moveElements(m_document->elementsBySection(HEADER3), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(ITEMS);
    moveElements(m_document->elementsBySection(ITEMS), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(SUM);
    moveElements(m_document->elementsBySection(SUM), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(DPH);
    moveElements(m_document->elementsBySection(DPH), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER1);
    posY = 297-m_document->marginBottom()-sheight;
    moveElements(m_document->elementsBySection(FOOTER1), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER2);
    posY -= sheight;
    moveElements(m_document->elementsBySection(FOOTER2), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER3);
    posY -= sheight;
    moveElements(m_document->elementsBySection(FOOTER3), posY, sheight);
    moveElements(m_document->elementsBySection(ANYSECTION), 0, 0);
}

void FakturaXmlPrint::Preview(bool print)
{
    FakturaXmlPrintout *print1 = new FakturaXmlPrintout(printTitle());
    print1->setDocument(m_document);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_document->pages());
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        FakturaXmlPrintout *print2 = new FakturaXmlPrintout(printTitle());
        print2->setDocument(m_document);
        print2->setHeaderLeftText(m_headerLeftText);
        print2->setHeaderCenterText(m_headerCenterText);
        print2->setHeaderRightText(m_headerRightText);
        print2->setHeaderLine(m_headerLine);
        print2->setMarginLeft(m_marginLeft);
        print2->setMarginRight(m_marginRight);
        print2->setMarginTop(m_marginTop);
        print2->setMarginBottom(m_marginBottom);
        print2->setPages(m_document->pages());
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTH,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void FakturaXmlPrint::Print()
{
    FakturaXmlPrintout *print = new FakturaXmlPrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    print->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

wxString FakturaXmlPrint::CreatePDF(const wxString &filename, const wxString &name, bool select)
{
    registerFontsForPdf();
    wxFileName fileName;
    if(select)
    {
        wxFileDialog dialog(m_parent, wxString::FromUTF8("Uložit fakturu jako"), wxGetHomeDir(), filename+wxT(".pdf"), wxT("PDF soubor (*.pdf)|*pdf"), wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
        if(dialog.ShowModal() == wxID_OK)
        {
            fileName.SetPath(dialog.GetDirectory());
            fileName.SetFullName(dialog.GetFilename());
        }
        else
        {
            return wxEmptyString;
        }
    }
    else
    {
        fileName.SetPath(wxStandardPaths::Get().GetUserDataDir());
        fileName.SetFullName(filename+wxT(".pdf"));
    }
    FakturaXmlPrintout *print = new FakturaXmlPrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintData data = print->printData();
    data.SetFilename(fileName.GetFullPath());
    wxPdfDC dc(data);
    dc.SetResolution(600);
    print->SetDC(&dc);
    print->OnPreparePrinting();
    print->OnBeginPrinting();
    int minPageNum = 1;
    int maxPageNum = m_document->pages();
    if(print->OnBeginDocument(minPageNum, maxPageNum))
    {
        dc.StartDoc("Start PDF...");
        wxPdfDocument *doc = dc.GetPdfDocument();
        if(!name.IsEmpty()) doc->SetTitle(name);
        doc->SetAuthor(idbmodel->firma().nazev());
        doc->SetCreator(wxString(FULLNAME));
        for(int pn = minPageNum; pn <= maxPageNum && print->HasPage(pn); pn++)
        {
            dc.StartPage();
            bool cont = print->OnPrintPage(pn);
            dc.EndPage();
            if(!cont) break;
        }
        print->OnEndDocument();
    }
    print->OnEndPrinting();
    return fileName.GetFullPath();
}

void FakturaXmlPrint::parseXml()
{
    Document doc;
    doc.readFromXml(m_xml);
    m_document->setFromDoc(doc);
    dxElementsArray elements = doc.elements();
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case LINE:
        {
            m_document->addElement(*static_cast<LineElement*>(elements[i]));
            break;
        }
        case RECTANGLE:
        {
            m_document->addElement(*static_cast<RectangleElement*>(elements[i]));
            break;
        }
        case IMAGE:
        {
            if(static_cast<ImageElement*>(elements[i])->path() == "{qr-platba}")
            {
                if(isFakturaForQrPlatba())
                {
                    m_document->addElement(ImageElement(static_cast<ImageElement*>(elements[i])->page(), qrtext(), static_cast<ImageElement*>(elements[i])->pos()));
                }
            }
            else
            {
                m_document->addElement(*static_cast<ImageElement*>(elements[i]));
            }
            break;
        }
        case TEXT:
        {
            TextElement el = *static_cast<TextElement*>(elements[i]);
            el.setValue(parseLine(el.value()));
            m_document->addElement(el);
            break;
        }
        case TEXTPARAGRAPH:
        {
            TextParagraphElement el = *static_cast<TextParagraphElement*>(elements[i]);
            wxArrayString valueo = el.value();
            wxArrayString value;
            for(size_t j=0; j<valueo.GetCount(); j++)
            {
                value.Add(parseLine(valueo[j]));
            }
            el.setValue(value);
            m_document->addElement(el);
            break;
        }
        case TABLEROW:
        {
            TableRowElement el = *static_cast<TableRowElement*>(elements[i]);
            wxString line = wxJoin(el.value(), ';');
            if(isRightPolozkaTag(line))
            {
                int height = 0;
                wxArrayString valueo = el.value();
                dxFaPolozkaArray polozky = idbmodel->fapolozkyByFakturaId(m_faktura.id());
                for(size_t k=0; k<polozky.GetCount(); k++)
                {
                    wxArrayString value;
                    for(size_t j=0; j<valueo.GetCount(); j++)
                    {
                        value.Add(parsePolozkaLine(valueo[j], polozky[k], idbmodel->hasDphSazba(polozky[k].sazbaid()) ? idbmodel->getDphSazba(polozky[k].sazbaid()).sazba() : 0));
                        value[j] = parseLine(value[j]);
                    }
                    if(!i)
                    {
                        el.setValue(value);
                        m_document->addElement(el);
                        height = el.height();
                    }
                    else
                    {
                        TableRowElement el1(el);
                        el1.setValue(value);
                        el1.setPos(wxPoint(el1.pos().x,el1.pos().y+height));
                        m_document->addElement(el1);
                        height += el1.height();
                    }
                }
            }
            else if(idbmodel->isFirmaPlatce() && isRightSazbaTag(line))
            {
                int height = 0;
                wxArrayString valueo = el.value();
                dxFaDphSazbaArray souhrn = idbmodel->dphFakturaSouhrn(m_faktura.id());
                for(size_t k=0; k<souhrn.GetCount(); k++)
                {
                    if(!dxutils::isSameDouble(souhrn[k].zaklad(),0.0))
                    {
                        wxArrayString value;
                        for(size_t j=0; j<valueo.GetCount(); j++)
                        {
                            value.Add(parseSazbaLine(valueo[j], souhrn[k].sazba(), souhrn[k].zaklad(), souhrn[k].dph()));
                            value[j] = parseLine(value[j]);
                        }
                        if(!i)
                        {
                            el.setValue(value);
                            m_document->addElement(el);
                            height = el.height();
                        }
                        else
                        {
                            TableRowElement el1(el);
                            el1.setValue(value);
                            el1.setPos(wxPoint(el1.pos().x,el1.pos().y+height));
                            m_document->addElement(el1);
                            height += el1.height();
                        }
                    }
                }
            }
            else
            {
                wxArrayString valueo = el.value();
                wxArrayString value;
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parseLine(valueo[j]));
                }
                el.setValue(value);
                m_document->addElement(el);
            }
            break;
        }
        default: ;
        }
    }
}

wxString FakturaXmlPrint::parseLine(wxString line)
{
    if(isRightFakturaTag(line))
    {
        line = parseFakturaLine(line);
    }
    if(isRightFirmaTag(line))
    {
        line = parseFirmaLine(line);
    }
    if(isRightKontaktTag(line))
    {
        line = parseKontaktLine(line);
    }
    return line;
}

wxString FakturaXmlPrint::parseFirmaLine(wxString line)
{
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    if(line.Contains("{firma-nazev}"))
    {
        line.Replace("{firma-nazev}", fi.nazev());
    }
    if(line.Contains("{firma-ulice}"))
    {
        line.Replace("{firma-ulice}",ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    }
    if(line.Contains("{firma-misto}"))
    {
        line.Replace("{firma-misto}",ad.isValid() ? ad.misto() : " ");
    }
    if(line.Contains("{firma-psc}"))
    {
        line.Replace("{firma-psc}",ad.isValid() ? ad.psc() : " ");
    }
    if(line.Contains("{firma-zeme}"))
    {
        line.Replace("{firma-zeme}",ad.isValid() ? ad.zeme() : " ");
    }
    if(line.Contains("{firma-registrace}"))
    {
        line.Replace("{firma-registrace}",fi.registrace());
    }
    if(line.Contains("{firma-ic}"))
    {
        line.Replace("{firma-ic}",fi.ic());
    }
    if(line.Contains("{firma-dic}"))
    {
        line.Replace("{firma-dic}",fi.dic());
    }
    if(line.Contains("{firma-telefon}"))
    {
        line.Replace("{firma-telefon}",fi.telefon());
    }
    if(line.Contains("{firma-email}"))
    {
        line.Replace("{firma-email}",fi.email());
    }
    if(line.Contains("{firma-www}"))
    {
        line.Replace("{firma-www}",fi.www());
    }
    return line;
}

wxString FakturaXmlPrint::parseFakturaLine(wxString line)
{
    FaUhrada uh = idbmodel->getFaUhrada(m_faktura.uhradaid());
    if(line.Contains("{faktura-cislo}"))
    {
        line.Replace("{faktura-cislo}", m_faktura.cislo());
    }
    if(line.Contains("{faktura-celkem}"))
    {
        line.Replace("{faktura-celkem}", wxString::Format("%.2f", m_faktura.celkem()));
    }
    if(line.Contains("{faktura-vystaveni}"))
    {
        line.Replace("{faktura-vystaveni}", m_faktura.vystaveni().FormatDate());
    }
    if(line.Contains("{faktura-splatnost}"))
    {
        line.Replace("{faktura-splatnost}",m_faktura.splatnost().FormatDate());
    }
    if(line.Contains("{faktura-duzp}"))
    {
        line.Replace("{faktura-duzp}",m_faktura.duzp().FormatDate());
    }
    if(line.Contains("{faktura-uhrada}"))
    {
        line.Replace("{faktura-uhrada}", uh.isValid() ? uh.uhrada() : " ");
    }
    if(line.Contains("{faktura-mena}"))
    {
        line.Replace("{faktura-mena}", idbmodel->zkratkaMena(m_faktura.menaid()));
    }
    if(line.Contains("{faktura-banka}"))
    {
        if(uh.isValid() && !uh.pokladnou())
        {
            Banka ba = idbmodel->getBanka(m_faktura.bankaid());
            line.Replace("{faktura-banka}", ba.isValid() ? wxString::Format("%s/%s", ba.cislouctu(), ba.kodbanky()): " ");
        }
        else
        {
            line.Replace("{faktura-banka}", " ");
        }
    }
    if(line.Contains("{faktura-vs}"))
    {
        line.Replace("{faktura-vs}", m_faktura.vs());
    }
    if(line.Contains("{faktura-ks}"))
    {
        line.Replace("{faktura-ks}", m_faktura.ks());
    }
    if(line.Contains("{faktura-ss}"))
    {
        line.Replace("{faktura-ss}", m_faktura.ss());
    }
    return line;
}

wxString FakturaXmlPrint::parseKontaktLine(wxString line)
{
    Kontakt ko = idbmodel->getKontakt(m_faktura.protistranaid());
    Adresa ad = idbmodel->getAdresa(ko.isValid() ? ko.adresaid() : -1);
    if(line.Contains("{kontakt-nazev}"))
    {
        line.Replace("{kontakt-nazev}", ko.isValid() ? ko.jmeno() : " ");
    }
    if(line.Contains("{kontakt-ulice}"))
    {
        line.Replace("{kontakt-ulice}", ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    }
    if(line.Contains("{kontakt-misto}"))
    {
        line.Replace("{kontakt-misto}", ad.isValid() ? ad.misto() : " ");
    }
    if(line.Contains("{kontakt-psc}"))
    {
        line.Replace("{kontakt-psc}", ad.isValid() ? ad.psc() : " ");
    }
    if(line.Contains("{kontakt-zeme}"))
    {
        line.Replace("{kontakt-zeme}", ad.isValid() ? ad.zeme() : " ");
    }
    if(line.Contains("{kontakt-ic}"))
    {
        line.Replace("{kontakt-ic}", ko.isValid() ? ko.ic() : " ");
    }
    if(line.Contains("{kontakt-dic}"))
    {
        line.Replace("{kontakt-dic}", ko.isValid() ? ko.dic() : " ");
    }
    return line;
}

wxString FakturaXmlPrint::parsePolozkaLine(wxString line, const FaPolozka &polozka, int sazba)
{
    if(line.Contains("{polozka-popis}"))
    {
        line.Replace("{polozka-popis}", polozka.popis());
    }
    if(line.Contains("{polozka-pocet}"))
    {
        line.Replace("{polozka-pocet}",wxString::Format("%.3f", polozka.pocet()));
    }
    if(line.Contains("{polozka-mj}"))
    {
        line.Replace("{polozka-mj}", polozka.mj());
    }
    if(line.Contains("{polozka-cenamj}"))
    {
        line.Replace("{polozka-cenamj}",wxString::Format("%.2f", dxutils::cenamj(polozka.cenamj(), polozka.bezdph(), polozka.sazbaid())));
    }
    if(line.Contains("{polozka-sazba}"))
    {
        line.Replace("{polozka-sazba}",wxString::Format("%d", sazba));
    }
    if(line.Contains("{polozka-dph}"))
    {
        line.Replace("{polozka-dph}",wxString::Format("%.2f", polozka.dph()));
    }
    if(line.Contains("{polozka-celkem}"))
    {
        line.Replace("{polozka-celkem}",wxString::Format("%.2f", polozka.celkem()));
    }
    return line;
}

wxString FakturaXmlPrint::parseSazbaLine(wxString line, int sazba, double zaklad, double dph)
{
    line.Replace("{dph-zaklad}", wxString::Format("%.2f", zaklad));
    line.Replace("{dph-dph}", wxString::Format("%.2f", dph));
    line.Replace("{dph-sazba}", wxString::Format("%d%%", sazba));
    return line;
}

void FakturaXmlPrint::moveElements(dxElementsArray elements, int posY, int sectionHeight)
{
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case LINE:
        {
            LineElement *el = static_cast<LineElement*>(elements[i]);
            if(el->anchor())
            {
                el->setStart(wxPoint(el->start().x, el->start().y+posY));
                el->setEnd(wxPoint(el->end().x, el->end().y+posY+sectionHeight));
                break;
            }
            if(el->borderTop())
            {
                el->setStart(wxPoint(el->start().x, posY));
                el->setEnd(wxPoint(el->end().x, posY));
                break;
            }
            if(el->borderBottom())
            {
                el->setStart(wxPoint(el->start().x, el->start().y+posY+sectionHeight));
                el->setEnd(wxPoint(el->end().x, el->end().y+posY+sectionHeight));
                break;
            }
            el->setStart(wxPoint(el->start().x, el->start().y+posY));
            el->setEnd(wxPoint(el->end().x, el->end().y+posY));
            break;
        }
        case RECTANGLE:
        {
            RectangleElement *el = static_cast<RectangleElement*>(elements[i]);
            if(el->anchor())
            {
                el->setStart(wxPoint(el->start().x, el->start().y+posY));
                el->setSize(wxSize(el->size().x, el->size().y+sectionHeight));
                break;
            }
            el->setStart(wxPoint(el->start().x, el->start().y+posY));
            break;
        }
        case IMAGE:
        {
            ImageElement *el = static_cast<ImageElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TEXT:
        {
            TextElement *el = static_cast<TextElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TEXTPARAGRAPH:
        {
            TextParagraphElement *el = static_cast<TextParagraphElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TABLEROW:
        {
            TableRowElement *el = static_cast<TableRowElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        default: ;
        }
    }
}

bool FakturaXmlPrint::isFakturaForQrPlatba()
{
    if(m_faktura.vydana())
    {
        FaUhrada uh = idbmodel->getFaUhrada(m_faktura.uhradaid());
        if(uh.isValid() && !uh.pokladnou())
        {
            if(!idbmodel->getBanka(m_faktura.bankaid()).iban().IsEmpty())
            {
                return true;
            }
        }
    }
    return false;
}

wxString FakturaXmlPrint::qrtext()
{
    wxString qrcode = "SPD*1.0";
    Banka ba = idbmodel->getBanka(m_faktura.bankaid());
    if(ba.bic().IsEmpty()) qrcode << "*ACC:" << ba.iban();
    else qrcode << "*ACC:" << ba.iban() << "+" << ba.bic();
    qrcode << "*AM:" << wxString::FromCDouble(m_faktura.celkem());
    qrcode << "*CC:" << idbmodel->kodMena(m_faktura.menaid());
    qrcode << "*DT:" << m_faktura.splatnost().Format("%Y%m%d");
    qrcode << "*X-VS:" << m_faktura.vs();
    if(!m_faktura.ss().IsEmpty()) qrcode << "*X-SS:" << m_faktura.ss();
    if(!m_faktura.ks().IsEmpty()) qrcode << "*X-KS:" << m_faktura.ks();
    return qrcode;
}

FakturaXmlPrintout::FakturaXmlPrintout(const wxString &title)
    : dxPrint(title), m_document(nullptr)
{
}

bool FakturaXmlPrintout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    int ydist = 0, xdist = 0;
    if(dc->IsKindOf(CLASSINFO(wxPdfDC)))
    {
        ydist = PAGE_OFFSET;
        xdist = PAGE_OFFSET;
    }
    drawHeader(dc, m_mmToLogical);
    if(m_document)
    {
        dxElementsArray elements = m_document->elementsByPage(page);
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            elements[i]->DrawToDC(dc, m_mmToLogical, ydist, xdist);
        }
    }
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void FakturaXmlPrintout::setDocument(Document *document)
{
    if(document)
    {
        m_document = document;
        m_footer = m_document->footer();
        m_footerText = m_document->footerText();
    }
}
