/***************************************************************
 * Name:      document.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include "document.h"

static int dxSortElementsBySize(Element ***e1, Element ***e2)
{
    if((**e1)->posY() == (**e2)->posY())
        return 0;
    if((**e1)->posY() < (**e2)->posY())
        return -1;
    return 1;
}

static int dxSortElements(Element ***e1, Element ***e2)
{
    if((**e1)->section() == (**e2)->section())
        return dxSortElementsBySize(e1, e2);
    if((**e1)->section() < (**e2)->section())
        return -1;
    return 1;
}

Document::Document()
    : m_style(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME)))),
      m_pages(1), m_marginLeft(10), m_marginRight(10), m_marginTop(10), m_marginBottom(10), m_footer(dxPrint::F_NUMBERING), m_footerText(""), m_headerLeftText(""), m_headerCenterText(""),
      m_headerRightText(""), m_headerLine(false)
{
}

dxElementsArray Document::elements() const
{
    return m_elements;
}

void Document::setElements(const dxElementsArray &elements)
{
    m_elements = elements;
}

dxElementsArray Document::elementsByPage(int page) const
{
    dxElementsArray elements;
    for(size_t i=0; i<m_elements.GetCount(); i++)
    {
        if(m_elements[i]->page() == page)
        {
            elements.Add(m_elements[i]);
        }
    }
    return elements;
}

dxElementsArray Document::elementsBySection(DOCSECTION section) const
{
    dxElementsArray elements;
    for(size_t i=0; i<m_elements.GetCount(); i++)
    {
        if(m_elements[i]->section() == section)
        {
            elements.Add(m_elements[i]);
        }
    }
    return elements;
}

size_t Document::getElementsCount() const
{
    return m_elements.GetCount();
}

void Document::addElement(const LineElement &element)
{
    if(element.page() > m_pages) m_pages = element.page();
    LineElement *pElement = new LineElement(element);
    m_elements.Add(pElement);
}

void Document::addElement(const TextElement &element)
{
    if(element.page() > m_pages) m_pages = element.page();
    TextElement *pElement = new TextElement(element);
    m_elements.Add(pElement);
}

void Document::addElement(const TextParagraphElement &element)
{
    if(element.page() > m_pages) m_pages = element.page();
    TextParagraphElement *pElement = new TextParagraphElement(element);
    m_elements.Add(pElement);
}

void Document::addElement(const TableRowElement &element)
{
    if(element.page() > m_pages) m_pages = element.page();
    TableRowElement *pElement = new TableRowElement(element);
    m_elements.Add(pElement);
}

void Document::addElement(const ImageElement &element)
{
    if(element.page() > m_pages) m_pages = element.page();
    ImageElement *pElement = new ImageElement(element);
    m_elements.Add(pElement);
}

void Document::addElement(const RectangleElement &element)
{
    if(element.page() > m_pages) m_pages = element.page();
    RectangleElement *pElement = new RectangleElement(element);
    m_elements.Add(pElement);
}

void Document::deleteElement(size_t index)
{
    if(index >= m_elements.GetCount()) return;
    m_elements.RemoveAt(index);
}

void Document::setFromDoc(const Document &doc)
{
    m_headerLeftText = doc.headerLeftText();
    m_headerCenterText = doc.headerCenterText();
    m_headerRightText = doc.headerRightText();
    m_headerLine = doc.headerLine();
    m_style = doc.style();
    m_footer = doc.footer();
    m_footerText = doc.footerText();
    m_pages = doc.pages();
    m_marginLeft = doc.marginLeft();
    m_marginTop = doc.marginTop();
    m_marginRight = doc.marginRight();
    m_marginBottom = doc.marginBottom();
}

int Document::sectionHeight(DOCSECTION section) const
{
    int sheight = 0;
    int eheight = 0;
    for(size_t i=0; i<m_elements.GetCount(); i++)
    {
        if(m_elements[i]->section() == section)
        {
            eheight = m_elements[i]->posY() + m_elements[i]->height();
            if(eheight > sheight) sheight = eheight;
        }
    }
    return sheight;
}

TextStyle Document::style() const
{
    return m_style;
}

void Document::setStyle(const TextStyle &style)
{
    m_style = style;
}

void Document::setFont(const wxFont &font)
{
    m_style.setFont(font);
}

void Document::setFontColour(const wxColour &fontColour)
{
    m_style.setFontColour(fontColour);
}

void Document::setBackColour(const wxColour &backColour)
{
    m_style.setBackColour(backColour);
}

int Document::pages() const
{
    return m_pages;
}

void Document::setPages(int pages)
{
    m_pages = pages;
}

void Document::setElementSelection(size_t index, bool select)
{
    if(index >= m_elements.GetCount()) return;
    m_elements[index]->setSelected(select);
}

void Document::clearElementSelection()
{
    for(size_t i=0; i<m_elements.GetCount(); i++)
    {
        m_elements[i]->setSelected(false);
    }
}

int Document::marginLeft() const
{
    return m_marginLeft;
}

void Document::setMarginLeft(int marginLeft)
{
    m_marginLeft = marginLeft;
}

int Document::marginRight() const
{
    return m_marginRight;
}

void Document::setMarginRight(int marginRight)
{
    m_marginRight = marginRight;
}

int Document::marginTop() const
{
    return m_marginTop;
}

void Document::setMarginTop(int marginTop)
{
    m_marginTop = marginTop;
}

int Document::marginBottom() const
{
    return m_marginBottom;
}

void Document::setMarginBottom(int marginBottom)
{
    m_marginBottom = marginBottom;
}

dxPrint::Footer Document::footer() const
{
    return m_footer;
}

void Document::setFooter(const dxPrint::Footer &footer)
{
    m_footer = footer;
}

wxString Document::footerText() const
{
    return m_footerText;
}

void Document::setFooterText(const wxString &footerText)
{
    m_footerText = footerText;
}

wxString Document::headerLeftText() const
{
    return m_headerLeftText;
}

void Document::setHeaderLeftText(const wxString &headerLeftText)
{
    m_headerLeftText = headerLeftText;
}

wxString Document::headerCenterText() const
{
    return m_headerCenterText;
}

void Document::setHeaderCenterText(const wxString &headerCenterText)
{
    m_headerCenterText = headerCenterText;
}

wxString Document::headerRightText() const
{
    return m_headerRightText;
}

void Document::setHeaderRightText(const wxString &headerRightText)
{
    m_headerRightText = headerRightText;
}

bool Document::headerLine() const
{
    return m_headerLine;
}

void Document::setHeaderLine(bool headerLine)
{
    m_headerLine = headerLine;
}

bool Document::saveToXml(const wxString &path)
{
    wxFileOutputStream fout(path);
    if(fout.IsOk())
    {
        wxXmlDocument xml;
        wxXmlNode *rootNode = new wxXmlNode(wxXML_ELEMENT_NODE, "dxdeDocument");
        wxXmlNode *marginsNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Margins");
        marginsNode->AddAttribute("Left", wxString::Format("%d", m_marginLeft));
        marginsNode->AddAttribute("Top", wxString::Format("%d", m_marginTop));
        marginsNode->AddAttribute("Bottom", wxString::Format("%d", m_marginBottom));
        marginsNode->AddAttribute("Right", wxString::Format("%d", m_marginRight));
        rootNode->AddChild(marginsNode);
        wxXmlNode *headerNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Header");
        headerNode->AddAttribute("Left", m_headerLeftText);
        headerNode->AddAttribute("Center", m_headerCenterText);
        headerNode->AddAttribute("Right", m_headerRightText);
        headerNode->AddAttribute("Line", wxString::Format("%d", m_headerLine));
        rootNode->AddChild(headerNode);
        wxXmlNode *footerNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Footer");
        footerNode->AddAttribute("Type", wxString::Format("%d", m_footer));
        footerNode->AddAttribute("Text", m_footerText);
        rootNode->AddChild(footerNode);
        rootNode->AddChild(m_style.createXmlNode());
        wxXmlNode *elementsNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Elements");
        for(size_t i=0; i<m_elements.GetCount(); i++)
        {
            elementsNode->AddChild(m_elements[i]->createXmlNode());
        }
        rootNode->AddChild(elementsNode);
        xml.SetRoot(rootNode);
        bool res = xml.Save(fout);
        fout.Close();
        return res;
    }
    else
        return false;
}

bool Document::readFromXml(const wxString &path)
{
    if(!wxFile::Exists(path)) return false;
    wxFileInputStream fin(path);
    if(fin.IsOk())
    {
        wxXmlDocument xml;
        if(!xml.Load(fin)) return false;
        if(xml.GetRoot()->GetName() != "dxdeDocument") return false;
        clearElements();
        wxXmlNode *rootNode = xml.GetRoot();
        wxXmlNode *marginsNode = rootNode->GetChildren();
        m_marginLeft = wxAtoi(marginsNode->GetAttribute("Left", "10"));
        m_marginTop = wxAtoi(marginsNode->GetAttribute("Top", "10"));
        m_marginBottom = wxAtoi(marginsNode->GetAttribute("Bottom", "10"));
        m_marginRight = wxAtoi(marginsNode->GetAttribute("Right", "10"));
        wxXmlNode *headerNode = marginsNode->GetNext();
        m_headerLeftText = headerNode->GetAttribute("Left");
        m_headerCenterText = headerNode->GetAttribute("Center");
        m_headerRightText = headerNode->GetAttribute("Right");
        m_headerLine = wxAtoi(headerNode->GetAttribute("Line", "0"));
        wxXmlNode *footerNode = headerNode->GetNext();
        m_footer = static_cast<dxPrint::Footer>(wxAtoi(footerNode->GetAttribute("Type", "0")));
        m_footerText = footerNode->GetAttribute("Text");
        wxXmlNode *styleNode = footerNode->GetNext();
        m_style.fromXmlNode(styleNode);
        wxXmlNode *elementsNode = styleNode->GetNext();
        wxXmlNode *elementNode = elementsNode->GetChildren();
        while(elementNode)
        {
            ELEMENTTYPE type = static_cast<ELEMENTTYPE>(wxAtoi(elementNode->GetAttribute("Type", "0")));
            Element *element = nullptr;
            switch(type) {
            case ELEMENT:
                break;
            case IMAGE:
                element = new ImageElement();
                break;
            case LINE:
                element = new LineElement();
                break;
            case TABLEROW:
                element = new TableRowElement();
                break;
            case TEXT:
                element = new TextElement();
                break;
            case TEXTPARAGRAPH:
                element = new TextParagraphElement();
                break;
            case RECTANGLE:
                element = new RectangleElement();
            }
            if(element)
            {
                if(element->fromXmlNode(elementNode))
                {
                    m_elements.Add(element);
                }
            }
            elementNode = elementNode->GetNext();
        }
        return true;
    }
    else
        return false;
}

Element *Document::element(size_t index)
{
    return m_elements.Item(index);
}

void Document::clearElements()
{
    m_elements.Clear();
}

void Document::sortElements()
{
    m_elements.Sort(dxSortElements);
}
