/***************************************************************
 * Name:      elements.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXELEMENTS_H
#define DXELEMENTS_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/dynarray.h>
#include "styles.h"
#include "qrcode/QrCode.hpp"

#define QR_PIXEL 1
#define QR_BORDER QR_PIXEL*4
#define QR_MEZERA QR_PIXEL*2

using namespace qrcodegen;

enum ELEMENTTYPE
{
    ELEMENT = 0,
    TEXT,
    TEXTPARAGRAPH,
    TABLEROW,
    IMAGE,
    LINE,
    RECTANGLE
};

enum DOCSECTION
{
    HEADER1 = 0,
    HEADER2,
    HEADER3,
    ITEMS,
    SUM,
    DPH,
    FOOTER3,
    FOOTER2,
    FOOTER1,
    ANYSECTION
};

class Element
{
public:
    Element();
    explicit Element(int page);
    virtual ~Element() {}

    virtual void DrawToDC(wxDC* dc, double mmToLogical, int ydist = 0, int xdist = 0);
    virtual ELEMENTTYPE elementType() { return ELEMENT; }
    virtual TextStyle style() const { return TextStyle(); }
    virtual wxString fontString() const { return ""; }
    virtual wxXmlNode *createXmlNode() const;
    virtual bool fromXmlNode(const wxXmlNode*) { return true; }
    int page() const;
    void setPage(int page);
    bool selected() const;
    void setSelected(bool selected);
    DOCSECTION section() const;
    void setSection(DOCSECTION section);
    virtual int height(bool inpx = false) const { return 0; }
    virtual int posY() const { return 0; }
protected:
    int m_page;
    bool m_selected;
    DOCSECTION m_section;
};

class LineElement : public Element
{
public:
    LineElement();
    LineElement(int page, wxPoint start, wxPoint end, LineStyle style=LineStyle());
    virtual ~LineElement() {}

    void DrawToDC(wxDC* dc, double mmToLogical, int dist = 0, int xdist = 0) override;
    ELEMENTTYPE elementType() override { return LINE; }
    wxPoint start() const;
    void setStart(const wxPoint &start);
    wxPoint end() const;
    void setEnd(const wxPoint &end);
    LineStyle lineStyle() const;
    void setLineStyle(const LineStyle &style);
    wxXmlNode *createXmlNode() const override;
    bool fromXmlNode(const wxXmlNode* node) override;
    bool anchor() const;
    void setAnchor(bool anchor);
    bool borderTop() const;
    void setBorderTop(bool borderTop);
    bool borderBottom() const;
    void setBorderBottom(bool borderBottom);
    int height(bool inpx = false) const override;
    int posY() const override;
private:
    wxPoint m_start;
    wxPoint m_end;
    LineStyle m_style;
    bool m_anchor; //ukotveni k sekci, tzn line se natahuje jak se sekce zvetsuje
    bool m_borderTop;
    bool m_borderBottom; //line je okrajem sekce
};

class RectangleElement : public Element
{
public:
    RectangleElement();
    RectangleElement(int page, wxPoint start, wxSize size, LineStyle style=LineStyle(), int fillet=0);
    virtual ~RectangleElement() {}

    void DrawToDC(wxDC* dc, double mmToLogical, int ydist = 0, int xdist = 0) override;
    ELEMENTTYPE elementType() override { return RECTANGLE; }
    wxPoint start() const;
    void setStart(const wxPoint &start);
    wxSize size() const;
    void setSize(const wxSize &size);
    LineStyle lineStyle() const;
    void setLineStyle(const LineStyle &style);
    wxXmlNode *createXmlNode() const override;
    bool fromXmlNode(const wxXmlNode* node) override;
    bool anchor() const;
    void setAnchor(bool anchor);
    int fillet() const;
    void setFillet(int fillet);
    int height(bool inpx = false) const override;
    int posY() const override;
private:
    wxPoint m_start;
    wxSize m_size;
    LineStyle m_style;
    bool m_anchor; //ukotveni k sekci, tzn rectangle se natahuje na vysku jak se sekce zvetsuje
    int m_fillet;
};

class TextElement : public Element
{
public:
    TextElement();
    TextElement(int page, const wxString &value, wxPoint pos, int width, wxTextAttrAlignment alignment=wxTEXT_ALIGNMENT_LEFT);
    TextElement(int page, const wxString &value, wxPoint pos, int width, wxTextAttrAlignment alignment, TextStyle style);
    virtual ~TextElement() override {}

    void DrawToDC(wxDC* dc, double mmToLogical, int ydist = 0, int xdist= 0) override;
    ELEMENTTYPE elementType() override { return TEXT; }
    wxString value() const;
    void setValue(const wxString &value);
    wxPoint pos() const;
    void setPos(const wxPoint &pos);
    int width() const;
    void setWidth(int width);
    wxTextAttrAlignment alignment() const;
    void setAlignment(const wxTextAttrAlignment &alignment);
    TextStyle style() const override;
    void setStyle(const TextStyle &style);
    wxString fontString() const override;
    int height(bool inpx = false) const override;
    wxXmlNode *createXmlNode() const override;
    bool fromXmlNode(const wxXmlNode* node) override;
    int posY() const override;
    bool forceSpace() const;
    void setForceSpace(bool forceSpace);
private:
    wxString m_value;
    wxPoint m_pos;
    int m_width;
    wxTextAttrAlignment m_alignment;
    TextStyle m_style;
    bool m_forceSpace;
};

class TextParagraphElement : public Element
{
public:
    TextParagraphElement();
    TextParagraphElement(int page, const wxArrayString &value, wxPoint pos, int width, wxTextAttrAlignment alignment=wxTEXT_ALIGNMENT_LEFT);
    TextParagraphElement(int page, const wxArrayString &value, wxPoint pos, int width, wxTextAttrAlignment alignment, TextStyle style);
    virtual ~TextParagraphElement() {}

    void DrawToDC(wxDC* dc, double mmToLogical, int ydist = 0, int xdist = 0) override;
    ELEMENTTYPE elementType() override { return TEXTPARAGRAPH; }
    wxArrayString value() const;
    void setValue(const wxArrayString &value);
    wxPoint pos() const;
    void setPos(const wxPoint &pos);
    int width() const;
    void setWidth(int width);
    wxTextAttrAlignment alignment() const;
    void setAlignment(const wxTextAttrAlignment &alignment);
    TextStyle style() const override;
    void setStyle(const TextStyle &style);
    wxString fontString() const override;
    int height(bool inpx = false) const override;
    bool border() const;
    void setBorder(bool border);
    LineStyle borderStyle() const;
    void setBorderStyle(const LineStyle &borderStyle);
    wxXmlNode *createXmlNode() const override;
    bool fromXmlNode(const wxXmlNode* node) override;
    int posY() const override;
private:
    wxArrayString m_value;
    wxPoint m_pos;
    int m_width;
    wxTextAttrAlignment m_alignment;
    TextStyle m_style;
    bool m_border;
    LineStyle m_borderStyle;
};

class TableRowElement : public Element
{
public:
    TableRowElement();
    TableRowElement(int page, const wxArrayString &value, wxPoint pos, wxArrayInt width);
    TableRowElement(int page, const wxArrayString &value, wxPoint pos, wxArrayInt width, wxArrayInt alignment);
    TableRowElement(int page, const wxArrayString &value, wxPoint pos, wxArrayInt width, wxArrayInt alignment, TextStyle style);
    virtual ~TableRowElement() {}

    void DrawToDC(wxDC* dc, double mmToLogical, int ydist = 0, int xdist = 0) override;
    ELEMENTTYPE elementType() override { return TABLEROW; }
    wxArrayString value() const;
    void setValue(const wxArrayString &value);
    wxPoint pos() const;
    void setPos(const wxPoint &pos);
    wxArrayInt width() const;
    void setWidth(const wxArrayInt &width);
    wxArrayInt alignment() const;
    void setAlignment(const wxArrayInt &alignment);
    TextStyle style() const override;
    void setStyle(const TextStyle &style);
    wxString fontString() const override;
    int height(bool inpx = false) const override;
    bool border() const;
    void setBorder(bool border);
    LineStyle borderStyle() const;
    void setBorderStyle(const LineStyle &borderStyle);
    int totalWidth();
    wxXmlNode *createXmlNode() const override;
    bool fromXmlNode(const wxXmlNode* node) override;
    wxString cellValue(size_t index) const;
    void setCellValue(size_t index, const wxString &value);
    void addCellValue(const wxString &value);
    int cellWidth(size_t index) const;
    void setCellWidth(size_t index, int width);
    void addCellWidth(int width);
    wxTextAttrAlignment cellAlignment(size_t index) const;
    void setCellAlignment(size_t index, wxTextAttrAlignment alignment);
    void addCellAlignment(wxTextAttrAlignment alignment);
    void deleteCell(size_t index);
    int posY() const override;
private:
    wxArrayString m_value;
    wxPoint m_pos;
    wxArrayInt m_width;
    wxArrayInt m_alignment;
    TextStyle m_style;
    bool m_border;
    LineStyle m_borderStyle;
};

class ImageElement : public Element
{
public:
    ImageElement();
    ImageElement(int page, const wxString &path, wxPoint pos, int width, wxTextAttrAlignment alignment=wxTEXT_ALIGNMENT_LEFT);
    ImageElement(int page, const wxString &code, wxPoint pos);
    virtual ~ImageElement(){}

    void DrawToDC(wxDC* dc, double mmToLogical, int ydist = 0, int xdist = 0) override;
    ELEMENTTYPE elementType() override { return IMAGE; }
    wxString path() const;
    void setPath(const wxString &path);
    wxPoint pos() const;
    void setPos(const wxPoint &pos);
    int width() const;
    void setWidth(int width);
    wxTextAttrAlignment alignment() const;
    void setAlignment(const wxTextAttrAlignment &alignment);
    int height(bool inpx = false) const override;
    wxSize imagePxSize() const;
    wxXmlNode *createXmlNode() const override;
    bool fromXmlNode(const wxXmlNode* node) override;
    int posY() const override;
    wxString qrcodetxt() const;
    void setQrcodetxt(const wxString &qrcodetxt);
    bool isqrcode() const;
    void setIsqrcode(bool isqrcode);
private:
    wxString m_path;
    wxPoint m_pos;
    int m_width;
    wxTextAttrAlignment m_alignment;
    wxString m_qrcodetxt;
    bool m_isqrcode;
};

WX_DECLARE_OBJARRAY(Element*, dxElementsArray);

#endif  /* DXELEMENTS_H */

