/***************************************************************
 * Name:      multitableprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/log.h>
#include <wx/tokenzr.h>
#include "multitableprint.h"

MultiTablePrint::MultiTablePrint()
    : m_headerLine(false), m_marginLeft(20-PAGE_OFFSET), m_marginRight(20+PAGE_OFFSET), m_marginTop(20-PAGE_OFFSET),
      m_marginBottom(20), m_pageWidth(210), m_pageHeight(289), m_paperOrientation(wxPORTRAIT), m_parent(nullptr), m_title("")
{
    m_tables = new dxTableArray();
}

MultiTablePrint::MultiTablePrint(wxWindow *parent, const wxString &title)
    : m_headerLeftText(wxEmptyString), m_headerCenterText(wxEmptyString),
      m_headerRightText(wxEmptyString), m_headerLine(false), m_marginLeft(20-PAGE_OFFSET), m_marginRight(20+PAGE_OFFSET), m_marginTop(20-PAGE_OFFSET),
      m_marginBottom(20), m_pageWidth(210), m_pageHeight(289), m_paperOrientation(wxPORTRAIT), m_parent(parent), m_title(title)
{
    m_tables = new dxTableArray();
}

void MultiTablePrint::Preview()
{
    int pages = 0;
    for(size_t i=0; i<m_tables->GetCount(); i++)
    {
        m_tables->Item(i).calculatePages(m_marginTop, m_marginBottom, m_pageHeight);
        pages += m_tables->Item(i).pages();
    }
    MultiTablePrintout *print1 = new MultiTablePrintout(printTitle());
    MultiTablePrintout *print2 = new MultiTablePrintout(printTitle());
    print1->setTable(m_tables);
    print1->setPaperOrientation(m_paperOrientation);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(pages);
    print1->performPageSetup();
    print2->setTable(m_tables);
    print2->setPaperOrientation(m_paperOrientation);
    print2->setHeaderLeftText(m_headerLeftText);
    print2->setHeaderCenterText(m_headerCenterText);
    print2->setHeaderRightText(m_headerRightText);
    print2->setHeaderLine(m_headerLine);
    print2->setMarginLeft(m_marginLeft);
    print2->setMarginRight(m_marginRight);
    print2->setMarginTop(m_marginTop);
    print2->setMarginBottom(m_marginBottom);
    print2->setPages(pages);
    print2->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(pages);
    data.SetFromPage(1);
    data.SetToPage(pages);
    wxPrintPreview *preview = new wxPrintPreview(print1, print2, &data);
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, m_paperOrientation==wxPORTRAIT?wxSize(PREVIEW_WIDTH,PREVIEW_HEIGHT):wxSize(PREVIEW_WIDTHL,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void MultiTablePrint::Print()
{
    int pages = 0;
    for(size_t i=0; i<m_tables->GetCount(); i++)
    {
        m_tables->Item(i).calculatePages(m_marginTop, m_marginBottom, m_pageHeight);
        pages += m_tables->Item(i).pages();
    }
    MultiTablePrintout *print = new MultiTablePrintout(printTitle());
    print->setPaperOrientation(m_paperOrientation);
    print->setTable(m_tables);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(pages);
    print->performPageSetup();
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(pages);
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

wxString MultiTablePrint::printTitle() const
{
    if(m_title.StartsWith(" "))
        return m_title;
    return wxString(" ")+m_title;
}

MultiTablePrintout::MultiTablePrintout(const wxString &title)
    : dxPrint(title), m_tables(nullptr)
{
}

bool MultiTablePrintout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    drawHeader(dc, m_mmToLogical);
    printPage(dc, getTableForPage(page), pageForTable(page), m_mmToLogical);
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void MultiTablePrintout::setTable(dxTableArray *tables)
{
    if(tables)
        m_tables = tables;
}

void MultiTablePrintout::printPage(wxDC *dc, const Table &table, int page, double mmToLogical)
{
    if(!table.isValid())
        return;
    int pageWidthMM, pageHeightMM;
    GetPageSizeMM(&pageWidthMM, &pageHeightMM);
    double topMarginLogical = mmToLogical*m_marginTop;
    wxSize titleSize(0,0);
    if(page == 1 && table.tableTitle().isValid())
    {
        TableTitle title = table.tableTitle();
        dc->SetFont(title.style().font());
        dc->SetTextForeground(title.style().fontColour());
        dc->SetTextBackground(title.style().backColour());
        titleSize = dc->GetMultiLineTextExtent(title.title());
        double xPos;
        switch(title.alignment()){
        case wxTEXT_ALIGNMENT_LEFT:
        {
            xPos = m_marginLeft*mmToLogical;
            break;
        }
        case wxTEXT_ALIGNMENT_RIGHT:
        {
            xPos = (pageWidthMM - m_marginRight)*mmToLogical - titleSize.x;
            break;
        }
        default:
        {
            xPos = (((((pageWidthMM - m_marginLeft - m_marginRight)/2.0)+m_marginLeft)*mmToLogical) - (titleSize.x/2.0));
        }
        }
        dc->DrawText(title.title(), static_cast<int>(xPos), static_cast<int>(topMarginLogical));
    }
    int tw = table.tableWidth();
    double rowY = topMarginLogical+titleSize.y;
    dxTableCellArray header = table.header();
    double x = 0.5*(pageWidthMM - tw);
    for(size_t i=0; i < header.GetCount(); i++)
    {
        drawCell(header[i], dc, x, rowY, table.columnWidth(i), mmToLogical);
        x += table.columnWidth(i);
    }
    rowY += table.headerHeight()*mmToLogical;
    TableRowsPerPage rows = table.rowsPerPage()[static_cast<size_t>(page)-1];
    for(int i=rows.rowStart(); i<=rows.rowEnd(); i++)
    {
        dxTableCellArray cells = table.cells(static_cast<size_t>(i));
        x = 0.5*(pageWidthMM - tw);
        for(size_t j=0; j<cells.GetCount(); j++)
        {
            drawCell(cells[j], dc, x, rowY, table.columnWidth(j), mmToLogical);
            x += table.columnWidth(j);
        }
        rowY += table.rowHeight(static_cast<size_t>(i))*mmToLogical;
    }
    drawGrid(dc, table, 0.5*(pageWidthMM - tw), topMarginLogical+titleSize.y, rows, mmToLogical);
}

void MultiTablePrintout::drawCell(TableCell cell, wxDC *dc, double x, double y, double cw, double mmToLogical)
{
    dc->SetFont(cell.style().font());
    dc->SetTextForeground(cell.style().fontColour());
    dc->SetTextBackground(cell.style().backColour());
    int textW, textH;
    wxString wrappedLine, line;
    wxArrayString lines = wxStringTokenize(cell.value(), wxT("\n") );
    wxArrayString wrappedLines;
    for(size_t l = 0; l < lines.GetCount(); l++)
    {
        line = lines[l];
        wrappedLine = "";
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); w++)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            dc->GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > 0.9*cw*mmToLogical && space)
            {
                wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                wrappedLine = wxT("");
                w = space;
            }
        }
        if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
    }
    for(size_t l = 0; l < wrappedLines.GetCount(); l++)
    {
        dc->GetTextExtent(wrappedLines[l], &textW, &textH);
        double cos = 0; // cell off-set
        switch(cell.alignment())
        {
        case wxTEXT_ALIGNMENT_LEFT:
            cos = CELL_TEXT_OFFSET*mmToLogical;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            cos = cw*mmToLogical - textW - CELL_TEXT_OFFSET*mmToLogical;
            break;
        default:
            cos = (cw*mmToLogical - textW) / 2;
        }
        dc->DrawText(wrappedLines[l], wxRound(x*mmToLogical+cos), wxRound(y));
        y += textH;
    }
}

void MultiTablePrintout::drawGrid(wxDC *dc, const Table &table, double x, double y, TableRowsPerPage rows, double mmToLogical)
{
    dc->SetPen(*wxBLACK_PEN);
    dc->DrawLine(wxRound(x*mmToLogical), wxRound(y), wxRound((x+table.tableWidth())*mmToLogical), wxRound(y));
    double h = table.headerHeight()*mmToLogical;
    dc->DrawLine(wxRound(x*mmToLogical), wxRound(y+h), wxRound((x+table.tableWidth())*mmToLogical), wxRound(y+h));
    for(int i=rows.rowStart(); i<=rows.rowEnd(); i++)
    {
        h += table.rowHeight(static_cast<size_t>(i))*mmToLogical;
        dc->DrawLine(wxRound(x*mmToLogical), wxRound(y+h), wxRound((x+table.tableWidth())*mmToLogical), wxRound(y+h));
    }
    dc->DrawLine(wxRound(x*mmToLogical), wxRound(y), wxRound(x*mmToLogical), wxRound(y+h));
    wxArrayDouble widths = table.columsWidth();
    for(size_t i=0; i<widths.GetCount(); i++)
    {
        x += wxRound(widths[i]);
        dc->DrawLine(wxRound(x*mmToLogical), wxRound(y), wxRound(x*mmToLogical), wxRound(y+h));
    }
}

Table MultiTablePrintout::getTableForPage(int page)
{
    int tpage = 0;
    for(size_t i=0; i<m_tables->GetCount(); i++)
    {
        tpage += m_tables->Item(i).pages();
        if(page <= tpage)
            return m_tables->Item(i);
    }
    return Table();
}

int MultiTablePrintout::pageForTable(int page)
{
    int result = page;
    for(size_t i=0; i<m_tables->GetCount(); i++)
    {
        if(result <= m_tables->Item(i).pages())
            break;
        else
            result -= m_tables->Item(i).pages();
    }
    return result;
}
