/***************************************************************
 * Name:      splittableprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/log.h>
#include <wx/tokenzr.h>
#include "splittableprint.h"

SplitTablePrint::SplitTablePrint(wxWindow *parent, const wxString &title)
    : m_table(nullptr), m_headerLeftText(wxEmptyString), m_headerCenterText(wxEmptyString),
      m_headerRightText(wxEmptyString), m_headerLine(false), m_marginLeft(20-PAGE_OFFSET), m_marginRight(20+PAGE_OFFSET), m_marginTop(20-PAGE_OFFSET),
      m_marginBottom(20), m_parent(parent), m_title(title)
{
}

void SplitTablePrint::Preview()
{
    m_table->calculatePages(m_marginTop, m_marginBottom, 289);
    SplitTablePrintout *print1 = new SplitTablePrintout(printTitle());
    SplitTablePrintout *print2 = new SplitTablePrintout(printTitle());
    print1->setTable(m_table);
    print2->setTable(m_table);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_table->pages());
    print2->setHeaderLeftText(m_headerLeftText);
    print2->setHeaderCenterText(m_headerCenterText);
    print2->setHeaderRightText(m_headerRightText);
    print2->setHeaderLine(m_headerLine);
    print2->setMarginLeft(m_marginLeft);
    print2->setMarginRight(m_marginRight);
    print2->setMarginTop(m_marginTop);
    print2->setMarginBottom(m_marginBottom);
    print2->setPages(m_table->pages());
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_table->pages());
    data.SetFromPage(1);
    data.SetToPage(m_table->pages());
    wxPrintPreview *preview = new wxPrintPreview(print1, print2, &data);
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTH,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void SplitTablePrint::Print()
{
    m_table->calculatePages(m_marginTop, m_marginBottom, 289);
    SplitTablePrintout *print = new SplitTablePrintout(printTitle());
    print->setTable(m_table);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_table->pages());
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_table->pages());
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

wxString SplitTablePrint::printTitle() const
{
    if(m_title.StartsWith(" "))
        return m_title;
    return wxString(" ")+m_title;
}

SplitTablePrintout::SplitTablePrintout(const wxString &title)
    : dxPrint(title), m_table(nullptr)
{
}

bool SplitTablePrintout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    drawHeader(dc, m_mmToLogical);
    printPage(dc, page, m_mmToLogical);
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void SplitTablePrintout::setTable(Table *table)
{
    if(table)
        m_table = table;
}

void SplitTablePrintout::printPage(wxDC *dc, int page, double mmToLogical)
{
    if(!m_table)
        return;
    if(!m_table->isValid())
        return;
    int pageWidthMM, pageHeightMM;
    GetPageSizeMM(&pageWidthMM, &pageHeightMM);
    double topMarginLogical = mmToLogical*m_marginTop;
    wxSize titleSize(0,0);
    if(page == 1 && m_table->tableTitle().isValid())
    {
        TableTitle title = m_table->tableTitle();
        dc->SetFont(title.style().font());
        dc->SetTextForeground(title.style().fontColour());
        dc->SetTextBackground(title.style().backColour());
        titleSize = dc->GetTextExtent(title.title());
        double xPos;
        switch(title.alignment()){
        case wxTEXT_ALIGNMENT_LEFT:
        {
            xPos = m_marginLeft*mmToLogical;
            break;
        }
        case wxTEXT_ALIGNMENT_RIGHT:
        {
            xPos = ((pageWidthMM - m_marginRight)*mmToLogical - titleSize.x);
            break;
        }
        default:
        {
            xPos = (((((pageWidthMM - m_marginLeft - m_marginRight)/2.0)+m_marginLeft)*mmToLogical) - (titleSize.x/2.0));
        }
        }
        dc->DrawText(title.title(), static_cast<int>(xPos), static_cast<int>(topMarginLogical));
    }
    size_t split = splitFromPage(page);
    int tw = m_table->tableWidthForSplit(split);
    TableColumnsSplit columnsSplit = m_table->columnsSplit()[split];
    double rowY = topMarginLogical+titleSize.y;
    dxTableCellArray header = m_table->header();
    double x = 0.5*(pageWidthMM - tw);
    for(size_t i=static_cast<size_t>(columnsSplit.columnStart()); i <= static_cast<size_t>(columnsSplit.columnEnd()); i++)
    {
        drawCell(header[i], dc, x, rowY, m_table->columnWidth(i), mmToLogical);
        x += m_table->columnWidth(i);
    }
    rowY += m_table->headerHeight()*mmToLogical;
    TableRowsPerPage rows = m_table->rowsPerPage()[(static_cast<size_t>(page)-1)/m_table->numberOfColumnsSplit()];
    for(int i=rows.rowStart(); i<=rows.rowEnd(); i++)
    {
        dxTableCellArray cells = m_table->cells(static_cast<size_t>(i));
        x = 0.5*(pageWidthMM - tw);
        for(size_t j=static_cast<size_t>(columnsSplit.columnStart()); j<static_cast<size_t>(columnsSplit.columnEnd()); j++)
        {
            drawCell(cells[j], dc, x, rowY, m_table->columnWidth(j), mmToLogical);
            x += m_table->columnWidth(j);
        }
        rowY += m_table->rowHeight(static_cast<size_t>(i))*mmToLogical;
    }
    drawGrid(dc, 0.5*(pageWidthMM - tw), topMarginLogical+titleSize.y, split, columnsSplit.columnStart(), columnsSplit.columnEnd(), rows, mmToLogical);
}

void SplitTablePrintout::drawCell(TableCell cell, wxDC *dc, double x, double y, double cw, double mmToLogical)
{
    dc->SetFont(cell.style().font());
    dc->SetTextForeground(cell.style().fontColour());
    dc->SetTextBackground(cell.style().backColour());
    int textW, textH;
    wxString wrappedLine, line;
    wxArrayString lines = wxStringTokenize(cell.value(), wxT("\n") );
    wxArrayString wrappedLines;
    for(size_t l = 0; l < lines.GetCount(); l++)
    {
        line = lines[l];
        wrappedLine = "";
        size_t space = 0;
        for(size_t w = 0; w < line.Len(); w++)
        {
            wrappedLine += line[w];
            if(line[w] == wxT(' ') || line[w] == wxT('\t')) space = w;
            dc->GetTextExtent(wrappedLine, &textW, &textH);
            if(textW > 0.9*cw*mmToLogical && space)
            {
                wrappedLines.Add(wrappedLine.Left(wrappedLine.Len()-w+space));
                wrappedLine = wxT("");
                w = space;
            }
        }
        if(wrappedLine != wxEmptyString) wrappedLines.Add(wrappedLine);
    }
    for(size_t l = 0; l < wrappedLines.GetCount(); l++)
    {
        dc->GetTextExtent(wrappedLines[l], &textW, &textH);
        double cos = 0; // cell off-set
        switch(cell.alignment())
        {
        case wxTEXT_ALIGNMENT_LEFT:
            cos = CELL_TEXT_OFFSET*mmToLogical;
            break;
        case wxTEXT_ALIGNMENT_RIGHT:
            cos = cw*mmToLogical - textW - CELL_TEXT_OFFSET*mmToLogical;
            break;
        default:
            cos = (cw*mmToLogical - textW) / 2;
        }
        dc->DrawText(wrappedLines[l], wxRound(x*mmToLogical+cos), wxRound(y));
        y += textH;
    }
}

void SplitTablePrintout::drawGrid(wxDC *dc, double x, double y, size_t split, int columnStart, int columnEnd, TableRowsPerPage rows, double mmToLogical)
{
    dc->SetPen(*wxBLACK_PEN);
    dc->DrawLine(wxRound(x*mmToLogical), wxRound(y), wxRound((x+m_table->tableWidthForSplit(split))*mmToLogical), wxRound(y));
    double h = m_table->headerHeight()*mmToLogical;
    dc->DrawLine(wxRound(x*mmToLogical), wxRound(y+h), wxRound((x+m_table->tableWidthForSplit(split))*mmToLogical), wxRound(y+h));
    for(int i=rows.rowStart(); i<=rows.rowEnd(); i++)
    {
        h += m_table->rowHeight(static_cast<size_t>(i))*mmToLogical;
        dc->DrawLine(wxRound(x*mmToLogical), wxRound(y+h), wxRound((x+m_table->tableWidthForSplit(split))*mmToLogical), wxRound(y+h));
    }
    dc->DrawLine(wxRound(x*mmToLogical), wxRound(y), wxRound(x*mmToLogical), wxRound(y+h));
    wxArrayDouble widths = m_table->columsWidth();
    for(size_t i=static_cast<size_t>(columnStart); i<=static_cast<size_t>(columnEnd); i++)
    {
        x += wxRound(widths[i]);
        dc->DrawLine(wxRound(x*mmToLogical), wxRound(y), wxRound(x*mmToLogical), wxRound(y+h));
    }
}

size_t SplitTablePrintout::splitFromPage(int page)
{
    size_t splits = m_table->numberOfColumnsSplit();
    size_t split = 0;
    for(int i=1; i<page; ++i)
    {
        split++;
        if(split+1 > splits)
            split = 0;
    }
    return split;
}
