/***************************************************************
 * Name:      wxMailer.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 *            Eran Ifrah
 * Copyright: 2017
 * License:   GPL3
 **************************************************************/

#include "config.h"
#include "wxMailer.h"
#include <wx/msgdlg.h>
#ifdef HAVE_CURL
    #include <curl/curl.h>
#endif

wxMailer::wxMailer(const wxString& username, const wxString& password, const wxString& smtpURL, bool useAuth, bool useSsl)
    : m_username(username)
    , m_password(password)
    , m_smtp(smtpURL)
    , m_useAuth(useAuth)
    , m_useSsl(useSsl)
{
}

wxMailer::~wxMailer() {}

static size_t payload_source(void* ptr, size_t size, size_t nmemb, void* userp)
{
    wxEmailMessage* message = static_cast<wxEmailMessage*>(userp);
    if(!message->HasMore()) {
        return 0;
    }

    if((size == 0) || (nmemb == 0) || ((size * nmemb) < 1)) {
        return 0;
    }

    size_t len = (message->Len() > (size * nmemb)) ? (size * nmemb) : message->Len();
    memcpy(ptr, message->AsChar(), len);
    message->Consume(len);
    return len;
}

bool wxMailer::Send(const wxEmailMessage& message)
{
    CURL* curl;
    curl = curl_easy_init();

    if(curl) {
        // Transport initialization
        if(m_useAuth)
        {
            curl_easy_setopt(curl, CURLOPT_USERNAME, m_username.mb_str(wxConvUTF8).data());
            curl_easy_setopt(curl, CURLOPT_PASSWORD, m_password.mb_str(wxConvUTF8).data());
        }
        curl_easy_setopt(curl, CURLOPT_URL, m_smtp.mb_str(wxConvUTF8).data());
        if(m_useSsl)
        {
            curl_easy_setopt(curl, CURLOPT_USE_SSL, static_cast<long>(CURLUSESSL_ALL));
            curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
            curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
        }

        // Message details
        curl_easy_setopt(curl, CURLOPT_MAIL_FROM, message.GetFromEmail().mb_str(wxConvUTF8).data());
        curl_slist* recipients = nullptr;
        recipients = curl_slist_append(recipients, message.GetToEmail().mb_str(wxConvUTF8).data());
        curl_easy_setopt(curl, CURLOPT_MAIL_RCPT, recipients);

        curl_easy_setopt(curl, CURLOPT_READFUNCTION, payload_source);
        curl_easy_setopt(curl, CURLOPT_READDATA, (void*)&message);
        curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
        CURLcode res = curl_easy_perform(curl);

        if(res != CURLE_OK) {
            wxMessageBox(curl_easy_strerror(res), "Email Error!", wxICON_ERROR | wxOK | wxCENTER);
            curl_slist_free_all(recipients);
            curl_easy_cleanup(curl);
            return false;
        }

        curl_slist_free_all(recipients);
        curl_easy_cleanup(curl);
    }
    return true;
}
